@@ -47,7 +47,6 @@
 #include "libguile/ports.h"
 #include "libguile/root.h"
 #include "libguile/fluids.h"
-#include "libguile/objects.h"
 #include "libguile/programs.h"
 
 #include "libguile/validate.h"
@@ -356,7 +355,7 @@ SCM_DEFINE (scm_procedure_source, "procedure-source", 1, 0, 0,
         }
     }
   case scm_tcs_struct:
-    if (!SCM_I_OPERATORP (proc))
+    if (!(SCM_OBJ_CLASS_FLAGS (proc) & SCM_CLASSF_PURE_GENERIC))
       break;
     goto procprop;
   case scm_tc7_smob:
