@@ -120,9 +120,6 @@ SCM_DEPRECATED SCM scm_unprotect_object (SCM obj);
   (SCM_SETCDR ((x), SCM_PACK (SCM_UNPACK (SCM_CDR (x)) | (y))))
 #define SCM_FREEP(x) (0)
 #define SCM_NFREEP(x) (1)
-#define SCM_GC8MARKP(x) SCM_GC_MARK_P (x)
-#define SCM_SETGC8MARK(x) SCM_SET_GC_MARK (x)
-#define SCM_CLRGC8MARK(x) SCM_CLEAR_GC_MARK (x)
 #define SCM_GCTYP16(x) SCM_TYP16 (x)
 #define SCM_GCCDR(x) SCM_CDR (x)
 SCM_DEPRECATED void scm_remember (SCM * ptr);
@@ -544,6 +541,12 @@ SCM_DEPRECATED SCM_STACKITEM *scm_i_stack_base (void);
 #define SCM_FLUIDP(x) scm_i_fluidp (x)
 SCM_DEPRECATED int scm_i_fluidp (SCM x);
 
+/* Deprecated in Guile 1.9.5 on 2009-11-15 because these are IPv4-only
+   functions which are deprecated upstream.  */
+
+SCM_DEPRECATED SCM scm_inet_aton (SCM address);
+SCM_DEPRECATED SCM scm_inet_ntoa (SCM inetid);
+
 /* In the old days, SCM_CRITICAL_SECTION_START stopped signal handlers
    from running, since in those days the handler directly ran scheme
    code, and that had to be avoided when the heap was not in a
