@@ -221,8 +221,7 @@ CEVAL (SCM x, SCM env)
    *
    * Even frames are eval frames, odd frames are apply frames.
    */
-  debug.vect = (scm_t_debug_info *) alloca (scm_debug_eframe_size
-					    * sizeof (scm_t_debug_info));
+  debug.vect = alloca (scm_debug_eframe_size * sizeof (scm_t_debug_info));
   debug.info = debug.vect;
   debug_info_end = debug.vect + scm_debug_eframe_size;
   scm_i_set_last_debug_frame (&debug);
@@ -304,7 +303,7 @@ dispatch:
           while (!scm_is_null (SCM_CDR (x)))
             {
               SCM test_result = EVALCAR (x, env);
-              if (scm_is_false (test_result) || SCM_NILP (test_result))
+              if (scm_is_false_or_nil (test_result))
                 RETURN (SCM_BOOL_F);
               else
                 x = SCM_CDR (x);
@@ -442,8 +441,7 @@ dispatch:
 		      xx = SCM_CDR (clause);
 		      proc = EVALCAR (xx, env);
 		      guard_result = SCM_APPLY (proc, arg1, SCM_EOL);
-		      if (scm_is_true (guard_result)
-			  && !SCM_NILP (guard_result))
+		      if (scm_is_true_and_not_nil (guard_result))
 			{
 			  proc = SCM_CDDR (xx);
 			  proc = EVALCAR (proc, env);
@@ -451,7 +449,7 @@ dispatch:
 			  goto apply_proc;
 			}
 		    }
-                  else if (scm_is_true (arg1) && !SCM_NILP (arg1))
+                  else if (scm_is_true_and_not_nil (arg1))
                     {
                       x = SCM_CDR (clause);
                       if (scm_is_null (x))
@@ -498,7 +496,7 @@ dispatch:
 
             SCM test_result = EVALCAR (test_form, env);
 
-            while (scm_is_false (test_result) || SCM_NILP (test_result))
+            while (scm_is_false_or_nil (test_result))
               {
                 {
                   /* Evaluate body forms.  */
@@ -552,7 +550,7 @@ dispatch:
           {
             SCM test_result = EVALCAR (x, env);
             x = SCM_CDR (x);  /* then expression */
-            if (scm_is_false (test_result) || SCM_NILP (test_result))
+            if (scm_is_false_or_nil (test_result))
               {
                 x = SCM_CDR (x);  /* else expression */
                 if (scm_is_null (x))
@@ -623,7 +621,7 @@ dispatch:
           while (!scm_is_null (SCM_CDR (x)))
             {
               SCM val = EVALCAR (x, env);
-              if (scm_is_true (val) && !SCM_NILP (val))
+              if (scm_is_true_and_not_nil (val))
                 RETURN (val);
               else
                 x = SCM_CDR (x);
@@ -735,13 +733,6 @@ dispatch:
 	case (ISYMNUM (SCM_IM_DELAY)):
 	  RETURN (scm_make_promise (scm_closure (SCM_CDR (x), env)));
 
-#if 0
-	  /* See futures.h for a comment why futures are not enabled.
-	   */
-	case (ISYMNUM (SCM_IM_FUTURE)):
-	  RETURN (scm_i_make_future (scm_closure (SCM_CDR (x), env)));
-#endif
-
 	  /* PLACEHOLDER for case (ISYMNUM (SCM_IM_DISPATCH)): The following
 	     code (type_dispatch) is intended to be the tail of the case
 	     clause for the internal macro SCM_IM_DISPATCH.  Please don't
@@ -752,139 +743,10 @@ dispatch:
 	   * (c.f. objects.c:scm_mcache_compute_cmethod) since that
 	   * cuts down execution time for type dispatch to 50%.  */
 	type_dispatch: /* inputs: x, arg1 */
-	  /* Type dispatch means to determine from the types of the function
-	   * arguments (i. e. the 'signature' of the call), which method from
-	   * a generic function is to be called.  This process of selecting
-	   * the right method takes some time.  To speed it up, guile uses
-	   * caching:  Together with the macro call to dispatch the signatures
-	   * of some previous calls to that generic function from the same
-	   * place are stored (in the code!) in a cache that we call the
-	   * 'method cache'.  This is done since it is likely, that
-	   * consecutive calls to dispatch from that position in the code will
-	   * have the same signature.  Thus, the type dispatch works as
-	   * follows: First, determine a hash value from the signature of the
-	   * actual arguments.  Second, use this hash value as an index to
-	   * find that same signature in the method cache stored at this
-	   * position in the code.  If found, you have also found the 
-	   * corresponding method that belongs to that signature.  If the
-	   * signature is not found in the method cache, you have to perform a
-	   * full search over all signatures stored with the generic
-	   * function.  */
-	{
-	    unsigned long int specializers;
-	    unsigned long int hash_value;
-	    unsigned long int cache_end_pos;
-	    unsigned long int mask;
-	    SCM method_cache;
-
-	    {
-	      SCM z = SCM_CDDR (x);
-	      SCM tmp = SCM_CADR (z);
-	      specializers = scm_to_ulong (SCM_CAR (z));
-
-	      /* Compute a hash value for searching the method cache.  There
-	       * are two variants for computing the hash value, a (rather)
-	       * complicated one, and a simple one.  For the complicated one
-	       * explained below, tmp holds a number that is used in the
-	       * computation.  */
-	      if (scm_is_simple_vector (tmp))
-		{
-		  /* This method of determining the hash value is much
-		   * simpler:  Set the hash value to zero and just perform a
-		   * linear search through the method cache.  */
-		  method_cache = tmp;
-		  mask = (unsigned long int) ((long) -1);
-		  hash_value = 0;
-		  cache_end_pos = SCM_SIMPLE_VECTOR_LENGTH (method_cache);
-		}
-	      else
-		{
-		  /* Use the signature of the actual arguments to determine
-		   * the hash value.  This is done as follows:  Each class has
-		   * an array of random numbers, that are determined when the
-		   * class is created.  The integer 'hashset' is an index into
-		   * that array of random numbers.  Now, from all classes that
-		   * are part of the signature of the actual arguments, the
-		   * random numbers at index 'hashset' are taken and summed
-		   * up, giving the hash value.  The value of 'hashset' is
-		   * stored at the call to dispatch.  This allows to have
-		   * different 'formulas' for calculating the hash value at
-		   * different places where dispatch is called.  This allows
-		   * to optimize the hash formula at every individual place
-		   * where dispatch is called, such that hopefully the hash
-		   * value that is computed will directly point to the right
-		   * method in the method cache.  */
-		  unsigned long int hashset = scm_to_ulong (tmp);
-		  unsigned long int counter = specializers + 1;
-		  SCM tmp_arg = arg1;
-		  hash_value = 0;
-		  while (!scm_is_null (tmp_arg) && counter != 0)
-		    {
-		      SCM class = scm_class_of (SCM_CAR (tmp_arg));
-		      hash_value += SCM_INSTANCE_HASH (class, hashset);
-		      tmp_arg = SCM_CDR (tmp_arg);
-		      counter--;
-		    }
-		  z = SCM_CDDR (z);
-		  method_cache = SCM_CADR (z);
-		  mask = scm_to_ulong (SCM_CAR (z));
-		  hash_value &= mask;
-		  cache_end_pos = hash_value;
-		}
-	    }
-
-	    {
-	      /* Search the method cache for a method with a matching
-	       * signature.  Start the search at position 'hash_value'.  The
-	       * hashing implementation uses linear probing for conflict
-	       * resolution, that is, if the signature in question is not
-	       * found at the starting index in the hash table, the next table
-	       * entry is tried, and so on, until in the worst case the whole
-	       * cache has been searched, but still the signature has not been
-	       * found.  */
-	      SCM z;
-	      do
-		{
-		  SCM args = arg1; /* list of arguments */
-		  z = SCM_SIMPLE_VECTOR_REF (method_cache, hash_value);
-		  while (!scm_is_null (args))
-		    {
-		      /* More arguments than specifiers => CLASS != ENV */
-		      SCM class_of_arg = scm_class_of (SCM_CAR (args));
-		      if (!scm_is_eq (class_of_arg, SCM_CAR (z)))
-			goto next_method;
-		      args = SCM_CDR (args);
-		      z = SCM_CDR (z);
-		    }
-		  /* Fewer arguments than specifiers => CAR != CLASS */
-                  if (!scm_is_pair (z))
-                    goto apply_vm_cmethod;
-                  else if (!SCM_CLASSP (SCM_CAR (z))
-                           && !scm_is_symbol (SCM_CAR (z)))
-                    goto apply_memoized_cmethod;
-		next_method:
-		  hash_value = (hash_value + 1) & mask;
-		} while (hash_value != cache_end_pos);
-
-	      /* No appropriate method was found in the cache.  */
-	      z = scm_memoize_method (x, arg1);
-
-              if (scm_is_pair (z))
-                goto apply_memoized_cmethod;
-              
-            apply_vm_cmethod:
-              proc = z;
-              PREP_APPLY (proc, arg1);
-              goto apply_proc;
-
-	    apply_memoized_cmethod: /* inputs: z, arg1 */
-              {
-                SCM formals = SCM_CMETHOD_FORMALS (z);
-                env = SCM_EXTEND_ENV (formals, arg1, SCM_CMETHOD_ENV (z));
-                x = SCM_CMETHOD_BODY (z);
-                goto nontoplevel_begin;
-              }
-	    }
+          {
+            proc = scm_mcache_compute_cmethod (x, arg1);
+            PREP_APPLY (proc, arg1);
+            goto apply_proc;
 	  }
 
 
@@ -1020,9 +882,11 @@ dispatch:
 #ifdef DEVAL
 	    if (scm_check_memoize_p && SCM_TRAPS_P)
 	      {
+		SCM arg1, retval;
+
 		SCM_CLEAR_TRACED_FRAME (debug);
-		SCM arg1 = scm_make_debugobj (&debug);
-		SCM retval = SCM_BOOL_T;
+		arg1 = scm_make_debugobj (&debug);
+		retval = SCM_BOOL_T;
 		SCM_TRAPS_P = 0;
 		retval = scm_call_4 (SCM_MEMOIZE_HDLR,
 				     scm_sym_memoize_symbol,
@@ -1164,22 +1028,22 @@ dispatch:
       case scm_tcs_struct:
 	if (SCM_OBJ_CLASS_FLAGS (proc) & SCM_CLASSF_PURE_GENERIC)
 	  {
-	    x = SCM_ENTITY_PROCEDURE (proc);
+	    x = SCM_GENERIC_METHOD_CACHE (proc);
 	    arg1 = SCM_EOL;
 	    goto type_dispatch;
 	  }
-	else if (SCM_I_OPERATORP (proc))
+#if 0
+	else if (SCM_I_ENTITYP (proc))
 	  {
 	    arg1 = proc;
-	    proc = (SCM_I_ENTITYP (proc)
-		    ? SCM_ENTITY_PROCEDURE (proc)
-		    : SCM_OPERATOR_PROCEDURE (proc));
+	    proc = SCM_ENTITY_PROCEDURE (proc);
 #ifdef DEVAL
 	    debug.info->a.proc = proc;
 	    debug.info->a.args = scm_list_1 (arg1);
 #endif
             goto evap1;
 	  }
+#endif
         else
           goto badfun;
       case scm_tc7_subr_1:
@@ -1293,7 +1157,7 @@ dispatch:
 	  case scm_tcs_struct:
 	    if (SCM_OBJ_CLASS_FLAGS (proc) & SCM_CLASSF_PURE_GENERIC)
 	      {
-		x = SCM_ENTITY_PROCEDURE (proc);
+		x = SCM_GENERIC_METHOD_CACHE (proc);
 #ifdef DEVAL
 		arg1 = debug.info->a.args;
 #else
@@ -1301,19 +1165,19 @@ dispatch:
 #endif
 		goto type_dispatch;
 	      }
-	    else if (SCM_I_OPERATORP (proc))
+#if 0
+	    else if (SCM_I_ENTITYP (proc))
 	      {
 		arg2 = arg1;
 		arg1 = proc;
-		proc = (SCM_I_ENTITYP (proc)
-			? SCM_ENTITY_PROCEDURE (proc)
-			: SCM_OPERATOR_PROCEDURE (proc));
+		proc = SCM_ENTITY_PROCEDURE (proc);
 #ifdef DEVAL
 		debug.info->a.args = scm_cons (arg1, debug.info->a.args);
 		debug.info->a.proc = proc;
 #endif
                 goto evap2;
 	      }
+#endif
             else
               goto badfun;
 	  case scm_tc7_subr_2:
@@ -1374,7 +1238,7 @@ dispatch:
 	  case scm_tcs_struct:
 	    if (SCM_OBJ_CLASS_FLAGS (proc) & SCM_CLASSF_PURE_GENERIC)
 	      {
-		x = SCM_ENTITY_PROCEDURE (proc);
+		x = SCM_GENERIC_METHOD_CACHE (proc);
 #ifdef DEVAL
 		arg1 = debug.info->a.args;
 #else
@@ -1382,19 +1246,16 @@ dispatch:
 #endif
 		goto type_dispatch;
 	      }
-	    else if (SCM_I_OPERATORP (proc))
+#if 0
+	    else if (SCM_I_ENTITYP (proc))
 	      {
 	      operatorn:
 #ifdef DEVAL
-		RETURN (SCM_APPLY (SCM_I_ENTITYP (proc)
-				   ? SCM_ENTITY_PROCEDURE (proc)
-				   : SCM_OPERATOR_PROCEDURE (proc),
+		RETURN (SCM_APPLY (SCM_ENTITY_PROCEDURE (proc),
 				   scm_cons (proc, debug.info->a.args),
 				   SCM_EOL));
 #else
-		RETURN (SCM_APPLY (SCM_I_ENTITYP (proc)
-				   ? SCM_ENTITY_PROCEDURE (proc)
-				   : SCM_OPERATOR_PROCEDURE (proc),
+		RETURN (SCM_APPLY (SCM_ENTITY_PROCEDURE (proc),
 				   scm_cons2 (proc, arg1,
 					      scm_cons (arg2,
 							scm_ceval_args (x,
@@ -1403,6 +1264,7 @@ dispatch:
 				   SCM_EOL));
 #endif
 	      }
+#endif
             else
               goto badfun;
 	  case scm_tc7_subr_0:
@@ -1609,11 +1471,13 @@ dispatch:
 #else
 	      arg1 = scm_cons2 (arg1, arg2, scm_ceval_args (x, env, proc));
 #endif
-	      x = SCM_ENTITY_PROCEDURE (proc);
+	      x = SCM_GENERIC_METHOD_CACHE (proc);
 	      goto type_dispatch;
 	    }
-	  else if (SCM_I_OPERATORP (proc))
+#if 0
+	  else if (SCM_I_ENTITYP (proc))
 	    goto operatorn;
+#endif
 	  else
 	    goto badfun;
 	case scm_tc7_subr_2:
@@ -1917,7 +1781,8 @@ tail:
 #endif
 	  RETURN (scm_apply_generic (proc, args));
 	}
-      else if (SCM_I_OPERATORP (proc))
+#if 0
+      else if (SCM_I_ENTITYP (proc))
 	{
 	  /* operator */
 #ifdef DEVAL
@@ -1926,9 +1791,7 @@ tail:
 	  args = (SCM_UNBNDP(arg1) ? SCM_EOL : scm_cons (arg1, args));
 #endif
 	  arg1 = proc;
-	  proc = (SCM_I_ENTITYP (proc)
-		  ? SCM_ENTITY_PROCEDURE (proc)
-		  : SCM_OPERATOR_PROCEDURE (proc));
+	  proc = SCM_ENTITY_PROCEDURE (proc);
 #ifdef DEVAL
 	  debug.vect[0].a.proc = proc;
 	  debug.vect[0].a.args = scm_cons (arg1, args);
@@ -1938,6 +1801,7 @@ tail:
 	  else
 	    goto badproc;
 	}
+#endif
       else
         goto badproc;
     default:
