@@ -44,9 +44,9 @@
 #include "libguile/keywords.h"
 #include "libguile/macros.h"
 #include "libguile/modules.h"
-#include "libguile/objects.h"
 #include "libguile/ports.h"
 #include "libguile/procprop.h"
+#include "libguile/programs.h"
 #include "libguile/random.h"
 #include "libguile/root.h"
 #include "libguile/smob.h"
@@ -54,12 +54,23 @@
 #include "libguile/strports.h"
 #include "libguile/vectors.h"
 #include "libguile/weaks.h"
+#include "libguile/vm.h"
 
 #include "libguile/validate.h"
 #include "libguile/goops.h"
 
 #define SPEC_OF(x)  SCM_SLOT (x, scm_si_specializers)
 
+#define SCM_CMETHOD_CODE(cmethod) SCM_CDR (cmethod)
+#define SCM_CMETHOD_FORMALS(cmethod) SCM_CAR (SCM_CMETHOD_CODE (cmethod))
+#define SCM_CMETHOD_BODY(cmethod) SCM_CDR (SCM_CMETHOD_CODE (cmethod))
+#define SCM_CMETHOD_ENV(cmethod)  SCM_CAR (cmethod)
+
+/* Port classes */
+#define SCM_IN_PCLASS_INDEX       0
+#define SCM_OUT_PCLASS_INDEX      SCM_I_MAX_PORT_TYPE_COUNT
+#define SCM_INOUT_PCLASS_INDEX    (2 * SCM_I_MAX_PORT_TYPE_COUNT)
+
 /* this file is a mess. in theory, though, we shouldn't have many SCM references
    -- most of the references should be to vars. */
 
@@ -105,10 +116,6 @@ SCM_VARIABLE (scm_var_make_extended_generic, "make-extended-generic");
    h1.
 */
 
-/* The following definition is located in libguile/objects.h:
-#define SCM_OBJ_CLASS_REDEF(x) (SCM_STRUCT_VTABLE_DATA(x)[scm_si_redefined])
-*/
-
 #define TEST_CHANGE_CLASS(obj, class)				       \
 	{							       \
 	  class = SCM_CLASS_OF (obj);				       \
@@ -145,7 +152,6 @@ SCM scm_class_extended_accessor;
 SCM scm_class_method;
 SCM scm_class_simple_method, scm_class_accessor_method;
 SCM scm_class_procedure_class;
-SCM scm_class_operator_class, scm_class_operator_with_setter_class;
 SCM scm_class_entity_class;
 SCM scm_class_number, scm_class_list;
 SCM scm_class_keyword;
@@ -282,11 +288,15 @@ SCM_DEFINE (scm_class_of, "class-of", 1, 0, 0,
 		return SCM_STRUCT_TABLE_CLASS (SCM_CDR (handle));
 	      else
 		{
-		  SCM name = SCM_STRUCT_TABLE_NAME (SCM_CDR (handle));
-		  SCM class = scm_make_extended_class_from_symbol (scm_is_true (name)
-						       ? name
-						       : scm_nullstr,
-						       SCM_I_OPERATORP (x));
+		  SCM class, name;
+
+		  name = SCM_STRUCT_TABLE_NAME (SCM_CDR (handle));
+		  if (!scm_is_symbol (name))
+		    name = scm_string_to_symbol (scm_nullstr);
+
+                  /* FIXME APPLICABLE structs */
+		  class =
+		    scm_make_extended_class_from_symbol (name, 0);
 		  SCM_SET_STRUCT_TABLE_CLASS (SCM_CDR (handle), class);
 		  return class;
 		}
@@ -750,7 +760,7 @@ SCM_DEFINE (scm_sys_inherit_magic_x, "%inherit-magic!", 2, 0, 0,
     }
   flags &= SCM_CLASSF_INHERIT;
 
-  if (! (flags & SCM_CLASSF_ENTITY))
+  if (! (flags & SCM_CLASSF_PURE_GENERIC))
     {
       long n = SCM_I_INUM (SCM_SLOT (class, scm_si_nfields));
 #if 0
@@ -765,8 +775,7 @@ SCM_DEFINE (scm_sys_inherit_magic_x, "%inherit-magic!", 2, 0, 0,
 #endif
       if (n > 0 && !(flags & SCM_CLASSF_METACLASS))
 	{
-	  /* NOTE: The following depends on scm_struct_i_size. */
-	  flags |= SCM_STRUCTF_LIGHT + n * sizeof (SCM); /* use light representation */
+	  flags |= SCM_STRUCTF_LIGHT; /* use light representation */
 	}
     }
   SCM_SET_CLASS_FLAGS (class, flags);
@@ -782,7 +791,7 @@ prep_hashsets (SCM class)
 {
   unsigned int i;
 
-  for (i = 0; i < 7; ++i)
+  for (i = 0; i < 8; ++i)
     SCM_SET_HASHSET (class, i, scm_c_uniform32 (goops_rstate));
 }
 
@@ -825,13 +834,8 @@ scm_basic_basic_make_class (SCM class, SCM name, SCM dsupers, SCM dslots)
   }
 
   /* Support for the underlying structs: */
-  SCM_SET_CLASS_FLAGS (z, (class == scm_class_entity_class
-			   ? (SCM_CLASSF_GOOPS_OR_VALID
-			      | SCM_CLASSF_OPERATOR
-			      | SCM_CLASSF_ENTITY)
-			   : class == scm_class_operator_class
-			   ? SCM_CLASSF_GOOPS_OR_VALID | SCM_CLASSF_OPERATOR
-			   : SCM_CLASSF_GOOPS_OR_VALID));
+  /* FIXME: set entity flag on z if class == entity_class ? */
+  SCM_SET_CLASS_FLAGS (z, SCM_CLASSF_GOOPS_OR_VALID);
   return z;
 }
 
@@ -1162,15 +1166,6 @@ SCM_DEFINE (scm_accessor_method_slot_definition, "accessor-method-slot-definitio
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (scm_sys_tag_body, "%tag-body", 1, 0, 0,
-	    (SCM body),
-	    "Internal GOOPS magic---don't use this function!")
-#define FUNC_NAME s_scm_sys_tag_body
-{
-  return scm_cons (SCM_IM_LAMBDA, body);
-}
-#undef FUNC_NAME
-
 /******************************************************************************
  *
  * S l o t   a c c e s s
@@ -1565,22 +1560,18 @@ SCM_DEFINE (scm_sys_allocate_instance, "%allocate-instance", 2, 0, 0,
 
   n = SCM_I_INUM (SCM_SLOT (class, scm_si_nfields));
 
-  /* Entities */
-  if (SCM_CLASS_FLAGS (class) & SCM_CLASSF_ENTITY)
+  /* FIXME applicable structs */
+  /* Generic functions */
+  if (SCM_CLASS_FLAGS (class) & SCM_CLASSF_PURE_GENERIC)
     {
+      SCM gf;
       m = (SCM *) scm_alloc_struct (n, scm_struct_entity_n_extra_words,
-				    "entity struct");
+				    "generic function");
       m[scm_struct_i_setter] = SCM_BOOL_F;
       m[scm_struct_i_procedure] = SCM_BOOL_F;
-      /* Generic functions */
-      if (SCM_CLASS_FLAGS (class) & SCM_CLASSF_PURE_GENERIC)
-	{
-	  SCM gf = wrap_init (class, m, n);
-	  clear_method_cache (gf);
-	  return gf;
-	}
-      else
-	return wrap_init (class, m, n);
+      gf = wrap_init (class, m, n);
+      clear_method_cache (gf);
+      return gf;
     }
 
   /* Class objects */
@@ -1595,10 +1586,7 @@ SCM_DEFINE (scm_sys_allocate_instance, "%allocate-instance", 2, 0, 0,
       for (i = scm_si_goops_fields; i < n; i++)
 	SCM_SET_SLOT (z, i, SCM_GOOPS_UNBOUND);
 
-      if (SCM_SUBCLASSP (class, scm_class_entity_class))
-	SCM_SET_CLASS_FLAGS (z, SCM_CLASSF_OPERATOR | SCM_CLASSF_ENTITY);
-      else if (SCM_SUBCLASSP (class, scm_class_operator_class))
-	SCM_SET_CLASS_FLAGS (z, SCM_CLASSF_OPERATOR);
+      /* FIXME propagate applicable struct flag */
 
       return z;
     }
@@ -1617,15 +1605,11 @@ SCM_DEFINE (scm_sys_set_object_setter_x, "%set-object-setter!", 2, 0, 0,
 #define FUNC_NAME s_scm_sys_set_object_setter_x
 {
   SCM_ASSERT (SCM_STRUCTP (obj)
-	      && ((SCM_CLASS_FLAGS (obj) & SCM_CLASSF_OPERATOR)
-		  || SCM_I_ENTITYP (obj)),
+              && (SCM_OBJ_CLASS_FLAGS (obj) & SCM_CLASSF_PURE_GENERIC),
 	      obj,
 	      SCM_ARG1,
 	      FUNC_NAME);
-  if (SCM_I_ENTITYP (obj))
-    SCM_SET_ENTITY_SETTER (obj, setter);
-  else
-    SCM_OPERATOR_CLASS (obj)->setter = setter;
+  SCM_SET_GENERIC_SETTER (obj, setter);
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -1650,12 +1634,13 @@ SCM_DEFINE (scm_sys_modify_instance, "%modify-instance", 2, 0, 0,
    */
   SCM_CRITICAL_SECTION_START;
   {
-    SCM car = SCM_CAR (old);
-    SCM cdr = SCM_CDR (old);
-    SCM_SETCAR (old, SCM_CAR (new));
-    SCM_SETCDR (old, SCM_CDR (new));
-    SCM_SETCAR (new, car);
-    SCM_SETCDR (new, cdr);
+    scm_t_bits word0, word1;
+    word0 = SCM_CELL_WORD_0 (old);
+    word1 = SCM_CELL_WORD_1 (old);
+    SCM_SET_CELL_WORD_0 (old, SCM_CELL_WORD_0 (new));
+    SCM_SET_CELL_WORD_1 (old, SCM_CELL_WORD_1 (new));
+    SCM_SET_CELL_WORD_0 (new, word0);
+    SCM_SET_CELL_WORD_1 (new, word1);
   }
   SCM_CRITICAL_SECTION_END;
   return SCM_UNSPECIFIED;
@@ -1672,13 +1657,14 @@ SCM_DEFINE (scm_sys_modify_class, "%modify-class", 2, 0, 0,
 
   SCM_CRITICAL_SECTION_START;
   {
-    SCM car = SCM_CAR (old);
-    SCM cdr = SCM_CDR (old);
-    SCM_SETCAR (old, SCM_CAR (new));
-    SCM_SETCDR (old, SCM_CDR (new));
+    scm_t_bits word0, word1;
+    word0 = SCM_CELL_WORD_0 (old);
+    word1 = SCM_CELL_WORD_1 (old);
+    SCM_SET_CELL_WORD_0 (old, SCM_CELL_WORD_0 (new));
+    SCM_SET_CELL_WORD_1 (old, SCM_CELL_WORD_1 (new));
     SCM_STRUCT_DATA (old)[scm_vtable_index_vtable] = SCM_UNPACK (old);
-    SCM_SETCAR (new, car);
-    SCM_SETCDR (new, cdr);
+    SCM_SET_CELL_WORD_0 (new, word0);
+    SCM_SET_CELL_WORD_1 (new, word1);
     SCM_STRUCT_DATA (new)[scm_vtable_index_vtable] = SCM_UNPACK (new);
   }
   SCM_CRITICAL_SECTION_END;
@@ -1788,6 +1774,150 @@ static SCM list_of_no_method;
 SCM_GLOBAL_SYMBOL (scm_sym_args, "args");
 
 
+/* The cache argument for scm_mcache_lookup_cmethod has one of two possible
+ * formats:
+ *
+ * Format #1:
+ * (SCM_IM_DISPATCH ARGS N-SPECIALIZED
+ *   #((TYPE1 ... . CMETHOD) ...)
+ *   GF)
+ *
+ * Format #2:
+ * (SCM_IM_HASH_DISPATCH ARGS N-SPECIALIZED HASHSET MASK
+ *   #((TYPE1 ... CMETHOD) ...)
+ *   GF)
+ *
+ * ARGS is either a list of expressions, in which case they
+ * are interpreted as the arguments of an application, or
+ * a non-pair, which is interpreted as a single expression
+ * yielding all arguments.
+ *
+ * SCM_IM_DISPATCH expressions in generic functions always
+ * have ARGS = the symbol `args' or the iloc #@0-0.
+ *
+ * We should probably not complicate this mechanism by
+ * introducing "optimizations" for getters and setters or
+ * primitive methods.  Getters and setter will normally be
+ * compiled into @slot-[ref|set!] or a procedure call.
+ * They rely on the dispatch performed before executing
+ * the code which contains them.
+ *
+ * We might want to use a more efficient representation of
+ * this form in the future, perhaps after we have introduced
+ * low-level support for syntax-case macros.
+ */
+
+SCM
+scm_mcache_lookup_cmethod (SCM cache, SCM args)
+{
+  unsigned long i, mask, n, end;
+  SCM ls, methods, z = SCM_CDDR (cache);
+  n = scm_to_ulong (SCM_CAR (z)); /* maximum number of specializers */
+  methods = SCM_CADR (z);
+
+  if (scm_is_simple_vector (methods))
+    {
+      /* cache format #1: prepare for linear search */
+      mask = -1;
+      i = 0;
+      end = SCM_SIMPLE_VECTOR_LENGTH (methods);
+    }
+  else
+    {
+      /* cache format #2: compute a hash value */
+      unsigned long hashset = scm_to_ulong (methods);
+      long j = n;
+      z = SCM_CDDR (z);
+      mask = scm_to_ulong (SCM_CAR (z));
+      methods = SCM_CADR (z);
+      i = 0;
+      ls = args;
+      if (!scm_is_null (ls))
+	do
+	  {
+	    i += SCM_STRUCT_DATA (scm_class_of (SCM_CAR (ls)))
+		 [scm_si_hashsets + hashset];
+	    ls = SCM_CDR (ls);
+	  }
+	while (j-- && !scm_is_null (ls));
+      i &= mask;
+      end = i;
+    }
+
+  /* Search for match  */
+  do
+    {
+      long j = n;
+      z = SCM_SIMPLE_VECTOR_REF (methods, i);
+      ls = args; /* list of arguments */
+      /* More arguments than specifiers => z = CMETHOD, not a pair.
+       * Fewer arguments than specifiers => CAR != CLASS or `no-method'.  */
+      if (!scm_is_null (ls) && scm_is_pair (z))
+	do
+	  {
+	    if (! scm_is_eq (scm_class_of (SCM_CAR (ls)), SCM_CAR (z)))
+	      goto next_method;
+	    ls = SCM_CDR (ls);
+	    z = SCM_CDR (z);
+	  }
+	while (j-- && !scm_is_null (ls) && scm_is_pair (z));
+      if (!scm_is_pair (z))
+	return z;
+    next_method:
+      i = (i + 1) & mask;
+    } while (i != end);
+  return SCM_BOOL_F;
+}
+
+SCM
+scm_mcache_compute_cmethod (SCM cache, SCM args)
+{
+  SCM cmethod = scm_mcache_lookup_cmethod (cache, args);
+  if (scm_is_false (cmethod))
+    /* No match - memoize */
+    return scm_memoize_method (cache, args);
+  return cmethod;
+}
+
+SCM
+scm_apply_generic (SCM gf, SCM args)
+{
+  SCM cmethod = scm_mcache_compute_cmethod (SCM_GENERIC_METHOD_CACHE (gf), args);
+  if (SCM_PROGRAM_P (cmethod))
+    return scm_vm_apply (scm_the_vm (), cmethod, args);
+  else if (scm_is_pair (cmethod))
+    return scm_eval_body (SCM_CDR (SCM_CMETHOD_CODE (cmethod)),
+                          SCM_EXTEND_ENV (SCM_CAR (SCM_CMETHOD_CODE (cmethod)),
+                                          args,
+                                          SCM_CMETHOD_ENV (cmethod)));
+  else
+    return scm_apply (cmethod, args, SCM_EOL);
+}
+
+SCM
+scm_call_generic_0 (SCM gf)
+{
+  return scm_apply_generic (gf, SCM_EOL);
+}
+
+SCM
+scm_call_generic_1 (SCM gf, SCM a1)
+{
+  return scm_apply_generic (gf, scm_list_1 (a1));
+}
+
+SCM
+scm_call_generic_2 (SCM gf, SCM a1, SCM a2)
+{
+  return scm_apply_generic (gf, scm_list_2 (a1, a2));
+}
+
+SCM
+scm_call_generic_3 (SCM gf, SCM a1, SCM a2, SCM a3)
+{
+  return scm_apply_generic (gf, scm_list_3 (a1, a2, a3));
+}
+
 SCM
 scm_make_method_cache (SCM gf)
 {
@@ -1803,7 +1933,7 @@ static void
 clear_method_cache (SCM gf)
 {
   SCM cache = scm_make_method_cache (gf);
-  SCM_SET_ENTITY_PROCEDURE (gf, cache);
+  SCM_SET_GENERIC_METHOD_CACHE (gf, cache);
   SCM_SET_SLOT (gf, scm_si_used_by, SCM_BOOL_F);
 }
 
@@ -1827,7 +1957,7 @@ SCM_DEFINE (scm_sys_invalidate_method_cache_x, "%invalidate-method-cache!", 1, 0
   {
     SCM n = SCM_SLOT (gf, scm_si_n_specialized);
     /* The sign of n is a flag indicating rest args. */
-    SCM_SET_MCACHE_N_SPECIALIZED (SCM_ENTITY_PROCEDURE (gf), n);
+    SCM_SET_MCACHE_N_SPECIALIZED (SCM_GENERIC_METHOD_CACHE (gf), n);
   }
   return SCM_UNSPECIFIED;
 }
@@ -2551,11 +2681,6 @@ create_standard_classes (void)
 	       scm_class_class, scm_class_class, SCM_EOL);
   make_stdcls (&scm_class_entity_class,    "<entity-class>",
 	       scm_class_class, scm_class_procedure_class, SCM_EOL);
-  make_stdcls (&scm_class_operator_class,  "<operator-class>",
-	       scm_class_class, scm_class_procedure_class, SCM_EOL);
-  make_stdcls (&scm_class_operator_with_setter_class,
-	       "<operator-with-setter-class>",
-	       scm_class_class, scm_class_operator_class, SCM_EOL);
   make_stdcls (&scm_class_method,	   "<method>",
 	       scm_class_class, scm_class_object,	   method_slots);
   make_stdcls (&scm_class_simple_method,   "<simple-method>",
@@ -2570,21 +2695,27 @@ create_standard_classes (void)
 	       scm_class_entity_class,
 	       scm_list_2 (scm_class_object, scm_class_applicable),
 	       SCM_EOL);
+  SCM_CLEAR_CLASS_FLAGS (scm_class_entity, SCM_STRUCTF_LIGHT);
   make_stdcls (&scm_class_entity_with_setter, "<entity-with-setter>",
 	       scm_class_entity_class, scm_class_entity,   SCM_EOL);
+  SCM_CLEAR_CLASS_FLAGS (scm_class_entity_with_setter, SCM_STRUCTF_LIGHT);
   make_stdcls (&scm_class_generic,	   "<generic>",
 	       scm_class_entity_class, scm_class_entity,   gf_slots);
+  SCM_CLEAR_CLASS_FLAGS (scm_class_generic, SCM_STRUCTF_LIGHT);
   SCM_SET_CLASS_FLAGS (scm_class_generic, SCM_CLASSF_PURE_GENERIC);
   make_stdcls (&scm_class_extended_generic, "<extended-generic>",
 	       scm_class_entity_class, scm_class_generic, egf_slots);
+  SCM_CLEAR_CLASS_FLAGS (scm_class_extended_generic, SCM_STRUCTF_LIGHT);
   SCM_SET_CLASS_FLAGS (scm_class_extended_generic, SCM_CLASSF_PURE_GENERIC);
   make_stdcls (&scm_class_generic_with_setter, "<generic-with-setter>",
 	       scm_class_entity_class,
 	       scm_list_2 (scm_class_generic, scm_class_entity_with_setter),
 	       SCM_EOL);
+  SCM_CLEAR_CLASS_FLAGS (scm_class_generic_with_setter, SCM_STRUCTF_LIGHT);
   SCM_SET_CLASS_FLAGS (scm_class_generic_with_setter, SCM_CLASSF_PURE_GENERIC);
   make_stdcls (&scm_class_accessor,	   "<accessor>",
 	       scm_class_entity_class, scm_class_generic_with_setter, SCM_EOL);
+  SCM_CLEAR_CLASS_FLAGS (scm_class_accessor, SCM_STRUCTF_LIGHT);
   SCM_SET_CLASS_FLAGS (scm_class_accessor, SCM_CLASSF_PURE_GENERIC);
   make_stdcls (&scm_class_extended_generic_with_setter,
 	       "<extended-generic-with-setter>",
@@ -2592,6 +2723,7 @@ create_standard_classes (void)
 	       scm_list_2 (scm_class_generic_with_setter,
 			   scm_class_extended_generic),
 	       SCM_EOL);
+  SCM_CLEAR_CLASS_FLAGS (scm_class_extended_generic_with_setter, SCM_STRUCTF_LIGHT);
   SCM_SET_CLASS_FLAGS (scm_class_extended_generic_with_setter,
 		       SCM_CLASSF_PURE_GENERIC);
   make_stdcls (&scm_class_extended_accessor, "<extended-accessor>",
@@ -2601,6 +2733,7 @@ create_standard_classes (void)
 	       SCM_EOL);
   fix_cpl (scm_class_extended_accessor,
 	   scm_class_extended_generic, scm_class_generic);
+  SCM_CLEAR_CLASS_FLAGS (scm_class_extended_accessor, SCM_STRUCTF_LIGHT);
   SCM_SET_CLASS_FLAGS (scm_class_extended_accessor, SCM_CLASSF_PURE_GENERIC);
 
   /* Primitive types classes */
@@ -2829,7 +2962,7 @@ make_struct_class (void *closure SCM_UNUSED,
   SCM sym = SCM_STRUCT_TABLE_NAME (data);
   if (scm_is_true (sym))
     {
-      int applicablep = SCM_CLASS_FLAGS (vtable) & SCM_CLASSF_OPERATOR;
+      int applicablep = 0; /* FIXME SCM_CLASS_FLAGS (vtable) & SCM_CLASSF_ENTITY */
 
       SCM_SET_STRUCT_TABLE_CLASS (data, 
 				  scm_make_extended_class_from_symbol (sym, applicablep));
@@ -2897,10 +3030,6 @@ scm_make_class (SCM meta, char *s_name, SCM supers, size_t size,
       SCM_SET_SLOT (class, scm_si_destructor, (SCM) destructor);
       SCM_SET_CLASS_DESTRUCTOR (class, scm_free_foreign_object);
     }
-  else if (size > 0)
-    {
-      SCM_SET_CLASS_INSTANCE_SIZE (class, size);
-    }
 
   SCM_SET_SLOT (class, scm_si_layout, scm_from_locale_symbol (""));
   SCM_SET_SLOT (class, scm_si_constructor, (SCM) constructor);
@@ -3000,7 +3129,7 @@ scm_wrap_component (SCM class, SCM container, void *data)
 					    obj,
 					    SCM_BOOL_F,
 					    scm_struct_ihashq,
-					    scm_sloppy_assq,
+					    (scm_t_assoc_fn) scm_sloppy_assq,
 					    0);
   SCM_SETCDR (handle, container);
   return obj;
