@@ -24,7 +24,6 @@
 
 #include "libguile/_scm.h"
 
-#include "libguile/objects.h"
 #include "libguile/strings.h"
 #include "libguile/vectors.h"
 #include "libguile/smob.h"
@@ -98,7 +97,7 @@ SCM_DEFINE (scm_procedure_p, "procedure?", 1, 0, 0,
     switch (SCM_TYP7 (obj))
       {
       case scm_tcs_struct:
-	if (!SCM_I_OPERATORP (obj))
+	if (!(SCM_OBJ_CLASS_FLAGS (obj) & SCM_CLASSF_PURE_GENERIC))
 	  break;
       case scm_tcs_closures:
       case scm_tcs_subrs:
@@ -144,16 +143,18 @@ SCM_DEFINE (scm_thunk_p, "thunk?", 1, 0, 0,
 	case scm_tc7_gsubr:
 	  return scm_from_bool (SCM_GSUBR_REQ (SCM_GSUBR_TYPE (obj)) == 0);
 	case scm_tc7_program:
-	  return scm_from_bool (SCM_PROGRAM_DATA (obj)->nargs == 0
-                                || (SCM_PROGRAM_DATA (obj)->nargs == 1
-                                    && SCM_PROGRAM_DATA (obj)->nrest));
+          {
+            int a, o, r;
+            if (scm_i_program_arity (obj, &a, &o, &r))
+              return scm_from_bool (a == 0);
+            else
+              return SCM_BOOL_F;
+          }
 	case scm_tc7_pws:
 	  obj = SCM_PROCEDURE (obj);
 	  goto again;
 	default:
-          if (SCM_PROGRAM_P (obj) && SCM_PROGRAM_DATA (obj)->nargs == 0)
-            return SCM_BOOL_T;
-          /* otherwise fall through */
+          return SCM_BOOL_F;
 	}
     }
   return SCM_BOOL_F;
@@ -260,7 +261,8 @@ SCM_DEFINE (scm_procedure, "procedure", 1, 0, 0,
     return SCM_PROCEDURE (proc);
   else if (SCM_STRUCTP (proc))
     {
-      SCM_ASSERT (SCM_I_OPERATORP (proc), proc, SCM_ARG1, FUNC_NAME);
+      SCM_ASSERT (SCM_OBJ_CLASS_FLAGS (proc) & SCM_CLASSF_PURE_GENERIC,
+                  proc, SCM_ARG1, FUNC_NAME);
       return proc;
     }
   SCM_WRONG_TYPE_ARG (1, proc);
@@ -279,11 +281,9 @@ scm_setter (SCM proc)
   else if (SCM_STRUCTP (proc))
     {
       SCM setter;
-      SCM_GASSERT1 (SCM_I_OPERATORP (proc),
+      SCM_GASSERT1 (SCM_OBJ_CLASS_FLAGS (proc) & SCM_CLASSF_PURE_GENERIC,
 		    g_setter, proc, SCM_ARG1, s_setter);
-      setter = (SCM_I_ENTITYP (proc)
-		? SCM_ENTITY_SETTER (proc)
-		: SCM_OPERATOR_SETTER (proc));
+      setter = SCM_GENERIC_SETTER (proc);
       if (SCM_NIMP (setter))
 	return setter;
       /* fall through */
