@@ -72,10 +72,8 @@
 
 #define STRINGBUF_HEADER_BYTES  (STRINGBUF_HEADER_SIZE * sizeof (SCM))
 
-#define STRINGBUF_F_SHARED      0x100
-#define STRINGBUF_F_WIDE        0x400 /* If true, strings have UCS-4
-                                         encoding.  Otherwise, strings
-                                         are Latin-1.  */
+#define STRINGBUF_F_SHARED      SCM_I_STRINGBUF_F_SHARED
+#define STRINGBUF_F_WIDE        SCM_I_STRINGBUF_F_WIDE
 
 #define STRINGBUF_TAG           scm_tc7_stringbuf
 #define STRINGBUF_SHARED(buf)   (SCM_CELL_WORD_0(buf) & STRINGBUF_F_SHARED)
@@ -88,10 +86,17 @@
 
 #define STRINGBUF_WIDE_CHARS(buf) ((scm_t_wchar *) STRINGBUF_CHARS (buf))
 
-#define SET_STRINGBUF_SHARED(buf) \
-  (SCM_SET_CELL_WORD_0 ((buf), SCM_CELL_WORD_0 (buf) | STRINGBUF_F_SHARED))
+#define SET_STRINGBUF_SHARED(buf)					\
+  do									\
+    {									\
+      /* Don't modify BUF if it's already marked as shared since it might be \
+	 a read-only, statically allocated stringbuf.  */		\
+      if (SCM_LIKELY (!STRINGBUF_SHARED (buf)))				\
+	SCM_SET_CELL_WORD_0 ((buf), SCM_CELL_WORD_0 (buf) | STRINGBUF_F_SHARED); \
+    }									\
+  while (0)
 
-#if SCM_STRING_LENGTH_HISTOGRAM
+#ifdef SCM_STRING_LENGTH_HISTOGRAM
 static size_t lenhist[1001];
 #endif
 
@@ -109,7 +114,7 @@ make_stringbuf (size_t len)
 
   SCM buf;
 
-#if SCM_STRING_LENGTH_HISTOGRAM
+#ifdef SCM_STRING_LENGTH_HISTOGRAM
   if (len < 1000)
     lenhist[len]++;
   else
@@ -135,7 +140,7 @@ make_wide_stringbuf (size_t len)
   SCM buf;
   size_t raw_len;
 
-#if SCM_STRING_LENGTH_HISTOGRAM
+#ifdef SCM_STRING_LENGTH_HISTOGRAM
   if (len < 1000)
     lenhist[len]++;
   else
@@ -235,7 +240,7 @@ scm_i_pthread_mutex_t stringbuf_write_mutex = SCM_I_PTHREAD_MUTEX_INITIALIZER;
 /* Read-only strings.
  */
 
-#define RO_STRING_TAG         (scm_tc7_string + 0x200)
+#define RO_STRING_TAG         scm_tc7_ro_string
 #define IS_RO_STRING(str)     (SCM_CELL_TYPE(str)==RO_STRING_TAG)
 
 /* Mutation-sharing substrings
@@ -967,7 +972,7 @@ SCM_DEFINE (scm_sys_symbol_dump, "%symbol-dump", 1, 0, 0, (SCM sym),
 }
 #undef FUNC_NAME
 
-#if SCM_STRING_LENGTH_HISTOGRAM
+#ifdef SCM_STRING_LENGTH_HISTOGRAM
 
 SCM_DEFINE (scm_sys_stringbuf_hist, "%stringbuf-hist", 0, 0, 0, (void), "")
 #define FUNC_NAME s_scm_sys_stringbuf_hist
@@ -1035,9 +1040,11 @@ SCM_DEFINE (scm_string, "string", 0, 0, 1,
 
   if (wide == 0)
     {
+      char *buf;
+
       result = scm_i_make_string (len, NULL);
       result = scm_i_string_start_writing (result);
-      char *buf = scm_i_string_writable_chars (result);
+      buf = scm_i_string_writable_chars (result);
       while (len > 0 && scm_is_pair (rest))
         {
           SCM elt = SCM_CAR (rest);
@@ -1050,9 +1057,11 @@ SCM_DEFINE (scm_string, "string", 0, 0, 1,
     }
   else
     {
+      scm_t_wchar *buf;
+
       result = scm_i_make_wide_string (len, NULL);
       result = scm_i_string_start_writing (result);
-      scm_t_wchar *buf = scm_i_string_writable_wide_chars (result);
+      buf = scm_i_string_writable_wide_chars (result);
       while (len > 0 && scm_is_pair (rest))
         {
           SCM elt = SCM_CAR (rest);
@@ -1824,8 +1833,8 @@ scm_i_deprecated_string_chars (SCM str)
      null-terminated.
   */
   if (IS_SH_STRING (str))
-    scm_misc_error (NULL, 
-		    "SCM_STRING_CHARS does not work with shared substrings.",
+    scm_misc_error (NULL,
+		    "SCM_STRING_CHARS does not work with shared substrings",
 		    SCM_EOL);
 
   /* We explicitly test for read-only strings to produce a better
@@ -1833,10 +1842,10 @@ scm_i_deprecated_string_chars (SCM str)
   */
 
   if (IS_RO_STRING (str))
-    scm_misc_error (NULL, 
-		    "SCM_STRING_CHARS does not work with read-only strings.",
+    scm_misc_error (NULL,
+		    "SCM_STRING_CHARS does not work with read-only strings",
 		    SCM_EOL);
-    
+
   /* The following is still wrong, of course...
    */
   str = scm_i_string_start_writing (str);
