@@ -63,9 +63,10 @@ SCM_DEFINE (scm_make_struct_layout, "make-struct-layout", 1, 0, 0,
 #define FUNC_NAME s_scm_make_struct_layout
 {
   SCM new_sym;
-  SCM_VALIDATE_STRING (1, fields);
   scm_t_wchar c;
 
+  SCM_VALIDATE_STRING (1, fields);
+
   { /* scope */
     size_t len;
     int x;
@@ -420,7 +421,7 @@ SCM_DEFINE (scm_make_struct, "make-struct", 2, 0, 1,
      need for a lock on the section below, as it does not access or update
      any globals, so the critical section has been removed. */
 
-  if (c_vtable[scm_struct_i_flags] & SCM_STRUCTF_ENTITY)
+  if (c_vtable[scm_struct_i_flags] & SCM_STRUCTF_GOOPS_HACK)
     {
       data = scm_alloc_struct (basic_size + tail_elts,
 			       scm_struct_entity_n_extra_words,
@@ -795,7 +796,7 @@ SCM_DEFINE (scm_struct_vtable_tag, "struct-vtable-tag", 1, 0, 0,
  */
 
 unsigned long
-scm_struct_ihashq (SCM obj, unsigned long n)
+scm_struct_ihashq (SCM obj, unsigned long n, void *closure)
 {
   /* The length of the hash table should be a relative prime it's not
      necessary to shift down the address.  */
@@ -809,7 +810,7 @@ scm_struct_create_handle (SCM obj)
 					    obj,
 					    SCM_BOOL_F,
 					    scm_struct_ihashq,
-					    scm_sloppy_assq,
+					    (scm_t_assoc_fn) scm_sloppy_assq,
 					    0);
   if (scm_is_false (SCM_CDR (handle)))
     SCM_SETCDR (handle, scm_cons (SCM_BOOL_F, SCM_BOOL_F));
