@@ -158,7 +158,7 @@ vm_dispatch_hook (struct scm_vm *vp, SCM hook, SCM hook_args)
     return;
   
   scm_dynwind_begin (0);
-  // FIXME, stack holder should be the vm
+  /* FIXME, stack holder should be the vm */
   vp->trace_frame = scm_c_make_vm_frame (SCM_BOOL_F, vp->fp, vp->sp, vp->ip, 0);
   scm_dynwind_unwind_handler (enfalsen_frame, vp, SCM_F_WIND_EXPLICITLY);
 
@@ -180,9 +180,8 @@ static SCM
 really_make_boot_program (long nargs)
 {
   SCM u8vec;
-  scm_t_uint8 text[] = { scm_op_mv_call, 0, 0, 1,
-                         scm_op_make_int8_1, scm_op_nop, scm_op_nop, scm_op_nop,
-                         scm_op_halt };
+  scm_t_uint8 text[] = { scm_op_mv_call, 0, 0, 0, 1,
+                         scm_op_make_int8_1, scm_op_halt };
   struct scm_objcode *bp;
   SCM ret;
 
@@ -192,12 +191,8 @@ really_make_boot_program (long nargs)
 
   bp = scm_malloc (sizeof (struct scm_objcode) + sizeof (text));
   memcpy (bp->base, text, sizeof (text));
-  bp->nargs = 0;
-  bp->nrest = 0;
-  bp->nlocs = 0;
   bp->len = sizeof(text);
   bp->metalen = 0;
-  bp->unused = 0;
 
   u8vec = scm_take_u8vector ((scm_t_uint8*)bp,
                              sizeof (struct scm_objcode) + sizeof (text));
@@ -303,11 +298,12 @@ make_vm (void)
 #define FUNC_NAME "make_vm"
 {
   int i;
+  struct scm_vm *vp;
 
   if (!scm_tc16_vm)
     return SCM_BOOL_F; /* not booted yet */
 
-  struct scm_vm *vp = scm_gc_malloc (sizeof (struct scm_vm), "vm");
+  vp = scm_gc_malloc (sizeof (struct scm_vm), "vm");
 
   vp->stack_size  = VM_DEFAULT_STACK_SIZE;
 
