@@ -28,8 +28,10 @@
   #:use-module (ice-9 receive)
   #:export (syntax-error 
             *current-language*
-            current-compilation-environment
-            compiled-file-name compile-file compile-and-load
+            compiled-file-name
+            compile-file
+            compile-and-load
+            read-and-compile
             compile
             decompile)
   #:export-syntax (call-with-compile-error-catch))
@@ -64,12 +66,6 @@
 (define (current-language)
   (fluid-ref *current-language*))
 
-(define *compilation-environment* (make-fluid))
-(define (current-compilation-environment)
-  "Return the current compilation environment (a module) or #f.  This
-function should only be called from stages in the compiler tower."
-  (fluid-ref *compilation-environment*))
-
 (define (call-once thunk)
   (let ((entered #f))
     (dynamic-wind
@@ -148,9 +144,9 @@ function should only be called from stages in the compiler tower."
 
 (define* (compile-file file #:key
                        (output-file #f)
-                       (env #f)
                        (from (current-language))
                        (to 'objcode)
+                       (env (default-environment from))
                        (opts '()))
   (let* ((comp (or output-file (compiled-file-name file)))
          (in (open-input-file file))
@@ -166,10 +162,11 @@ function should only be called from stages in the compiler tower."
       file)
     comp))
 
-(define* (compile-and-load file #:key (from 'scheme) (to 'value) (opts '()))
+(define* (compile-and-load file #:key (from 'scheme) (to 'value)
+                           (env (current-module)) (opts '()))
   (read-and-compile (open-input-file file)
                     #:from from #:to to #:opts opts
-                    #:env (current-module)))
+                    #:env env))
 
 
 ;;;
@@ -198,48 +195,24 @@ function should only be called from stages in the compiler tower."
           (else
            (lp (cdr in) (caar in))))))
 
-(define (make-compilation-module)
-  "Return a fresh module to be used as the compilation environment."
-
-  ;; Ideally we'd duplicate the whole module hierarchy so that `set!',
-  ;; `fluid-set!', etc. don't have any effect in the current environment.
-
-  (let ((m (make-module)))
-    (beautify-user-module! m)
-
-    ;; Provide a separate `current-reader' fluid so that the Scheme language
-    ;; reader doesn't get to see the REPL's settings for `current-reader',
-    ;; which would lead to an infinite loop.
-    (module-define! m 'current-reader (make-fluid))
-
-    m))
-
-(define (language-default-environment lang)
-  "Return the default compilation environment for source language LANG."
-  (if (or (eq? lang 'scheme)
-          (eq? lang (lookup-language 'scheme)))
-      (make-compilation-module)
-      #f))
-
 (define* (read-and-compile port #:key
-                           (env #f)
                            (from (current-language))
                            (to 'objcode)
+                           (env (default-environment from))
                            (opts '()))
   (let ((from (ensure-language from))
         (to (ensure-language to)))
     (let ((joint (find-language-joint from to)))
-      (with-fluids ((*current-language* from)
-                    (*compilation-environment*
-                     (or env
-                         (language-default-environment from))))
-        (let lp ((exps '()) (env #f)
-                 (cenv (fluid-ref *compilation-environment*)))
-          (let ((x ((language-reader (current-language)) port)))
+      (with-fluids ((*current-language* from))
+        (let lp ((exps '()) (env #f) (cenv env))
+          (let ((x ((language-reader (current-language)) port cenv)))
             (cond
              ((eof-object? x)
               (compile ((language-joiner joint) (reverse exps) env)
-                       #:from joint #:to to #:env env #:opts opts))
+                       #:from joint #:to to
+                       ;; env can be false if no expressions were read.
+                       #:env (or env (default-environment joint))
+                       #:opts opts))
              (else
               ;; compile-fold instead of compile so we get the env too
               (receive (jexp jenv jcenv)
@@ -248,9 +221,9 @@ function should only be called from stages in the compiler tower."
                 (lp (cons jexp exps) jenv jcenv))))))))))
 
 (define* (compile x #:key
-                  (env #f)
                   (from (current-language))
                   (to 'value)
+                  (env (default-environment from))
                   (opts '()))
 
   (let ((warnings (memq #:warnings opts)))
@@ -263,9 +236,7 @@ function should only be called from stages in the compiler tower."
                     warnings))))
 
   (receive (exp env cenv)
-      (let ((env (or env (language-default-environment from))))
-        (with-fluids ((*compilation-environment* env))
-          (compile-fold (compile-passes from to opts) x env opts)))
+      (compile-fold (compile-passes from to opts) x env opts)
     exp))
 
 
