@@ -1,35 +1,39 @@
 ;;; Guile VM frame functions
 
-;;; Copyright (C) 2001, 2009 Free Software Foundation, Inc.
-;;; Copyright (C) 2005 Ludovic Courts  <ludovic.courtes@laas.fr>
+;;; Copyright (C) 2001, 2005, 2009 Free Software Foundation, Inc.
 ;;;
-;;; This program is free software; you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by
-;;; the Free Software Foundation; either version 2 of the License, or
-;;; (at your option) any later version.
+;;; This library is free software; you can redistribute it and/or
+;;; modify it under the terms of the GNU Lesser General Public
+;;; License as published by the Free Software Foundation; either
+;;; version 3 of the License, or (at your option) any later version.
 ;;;
-;;; This program is distributed in the hope that it will be useful,
+;;; This library is distributed in the hope that it will be useful,
 ;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
+;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;;; Lesser General Public License for more details.
 ;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with this program; if not, write to the Free Software
-;;; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+;;; You should have received a copy of the GNU Lesser General Public
+;;; License along with this library; if not, write to the Free Software
+;;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 ;;; Code:
 
 (define-module (system vm frame)
+  #:use-module (system base pmatch)
   #:use-module (system vm program)
   #:use-module (system vm instruction)
+  #:use-module (system vm objcode)
   #:use-module ((srfi srfi-1) #:select (fold))
   #:export (vm-frame?
             vm-frame-program
             vm-frame-local-ref vm-frame-local-set!
+            vm-frame-instruction-pointer
             vm-frame-return-address vm-frame-mv-return-address
             vm-frame-dynamic-link
-            vm-frame-stack
+            vm-frame-num-locals
 
+            vm-frame-bindings vm-frame-binding-ref vm-frame-binding-set!
+            vm-frame-arguments
 
             vm-frame-number vm-frame-address
             make-frame-chain
@@ -44,6 +48,63 @@
 
 (load-extension "libguile" "scm_init_frames")
 
+(define (vm-frame-bindings frame)
+  (map (lambda (b)
+         (cons (binding:name b) (binding:index b)))
+       (program-bindings-for-ip (vm-frame-program frame)
+                                (vm-frame-instruction-pointer frame))))
+
+(define (vm-frame-binding-set! frame var val)
+  (let ((i (assq-ref (vm-frame-bindings frame) var)))
+    (if i
+        (vm-frame-local-set! frame i val)
+        (error "variable not bound in frame" var frame))))
+
+(define (vm-frame-binding-ref frame var)
+  (let ((i (assq-ref (vm-frame-bindings frame) var)))
+    (if i
+        (vm-frame-local-ref frame i)
+        (error "variable not bound in frame" var frame))))
+
+;; Basically there are two cases to deal with here:
+;;
+;;   1. We've already parsed the arguments, and bound them to local
+;;      variables. In a standard (lambda (a b c) ...) call, this doesn't
+;;      involve any argument shuffling; but with rest, optional, or
+;;      keyword arguments, the arguments as given to the procedure may
+;;      not correspond to what's on the stack. We reconstruct the
+;;      arguments using e.g. for the case above: `(,a ,b ,c). This works
+;;      for rest arguments too: (a b . c) => `(,a ,b . ,c)
+;;
+;;   2. We have failed to parse the arguments. Perhaps it's the wrong
+;;      number of arguments, or perhaps we're doing a typed dispatch and
+;;      the types don't match. In that case the arguments are all on the
+;;      stack, and nothing else is on the stack.
+(define (vm-frame-arguments frame)
+  (cond
+   ((program-lambda-list (vm-frame-program frame)
+                         (vm-frame-instruction-pointer frame))
+    ;; case 1
+    => (lambda (formals)
+         (let lp ((formals formals))
+           (pmatch formals
+             (() '())
+             ((,x . ,rest) (guard (symbol? x))
+              (cons (vm-frame-binding-ref frame x) (lp rest)))
+             ((,x . ,rest)
+              ;; could be a keyword
+              (cons x (lp rest)))
+             (,rest (guard (symbol? rest))
+              (vm-frame-binding-ref frame rest))
+             ;; let's not error here, as we are called during
+             ;; backtraces...
+             (else '???)))))
+   (else
+    ;; case 2
+    (map (lambda (i)
+           (vm-frame-local-ref frame i))
+         (iota (vm-frame-num-locals frame))))))
+
 ;;;
 ;;; Frame chain
 ;;;
